/***************************** SCARAB Header File **************************/
/*  This is header file contains all the prototypes and all the varaibles. */
/*  File name : scarab.h
    Date      : 2/16/95                                                   */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <malloc.h>

#define CM_5 1
#define Paragon 2
#define Delta 3

#include "Machine.h"
#if MACHINE == CM_5
    #include  <cm/cmmd.h>   /* CM-5 Labriry */
#elif MACHINE == Paragon
  #include  <nx.h>       /* Paragon Labriry */
#elif MACHINE == Delta
  #include <mesh.h> 
#endif




/*===================  Global Constants  =======================*/



#define SUCCESS 1
#define ANY_SOURCE 0
#define ANY_TAG 0

long Status;
int Error;
int ArgTypeCheck;   /* 1 -> trun on the checking, 0 -> trun off the checking*/
int Self;   /* Processor address */
int PS;     /* Partation size */

/*******************     Expert System Prototype     ***************/
int Expert_System(void);



/********************   Random Access Routines     ***********/

/*--- Random Access Write   ----*/
/*--- for Integers ,testing  ----*/
/* Each processor has N/PS values, and  N/PS elements . So the
total number of elements is N. */
int RAW(
int  *,   /*  List of N/PS values to be written to some global/local elements */
int  *,  /*   List of N/PS indices, one for each value. These indices are global */
int  ,    /*  This is the number of values.*/
int  ,    /* Number of elements */
int  *,  /*  List of M/PS elements */
int  ,  /* The algorithm */ 
int  ,  /* algorithm 1 */
int  ); /* algorithm 2 */

/******************** Collectives Communications routines prototypes *****/

void preprocessing_comm(
int *,
int *,
int *,
int *,
int *,
int *);




/*-- All-to-all with uniform message sizes --*/
/* Each proceesor send a block to each processor, such that processor i send
   the jth block to processor j, and processor j receive that block and
   store it in the ith block.                                           */

int AlltoAll(
  void *,    /* address of the send buffer */
  int   ,   /* Number of elements send/receive by each processor */
  int   ,   /* the size , in bytes, of the elements */
  void *,   /* address of the receive buffer */
  int   );  /* the algorithm number , if any*/

/*--  LP_Xor : Implementation of All-to-All ----*/
int LP_All(
  void *,    /* address of the send buffer */
  int   ,   /* Number of elements send/receive by each processor */
  int   ,   /* the size , in bytes, of the elements */
  void *);  /* address of the receive buffer */

/*--  Async. Comm. : Implementation of All-to-All ----*/
int AC_All(
  void *,    /* address of the send buffer */
  int   ,   /* Number of elements send/receive by each processor */
  int   ,   /* the size , in bytes, of the elements */
  void *);  /* address of the receive buffer */


/*--- Distributed Scheduling : Implementation of All-to-All ------*/
int  DS_All(
  void *,    /* address of the send buffer */
  int   ,   /* Number of elements send/receive by each processor */
  int   ,   /* the size , in bytes, of the elements */
  void *);  /* address of the receive buffer */
         /*====================================================*/
/*-- Many-to-Many with un-uniform message sizes and contigous buffer--*/
/* Each proceesor send a block to each processor, such that processor i send
   the  block at sdispls[j] of size scount[j] to processor j, and processor j receive that block and store it in the block at rdispls[[i]. Notice that :
i's scount[j] must equal to j`s rcount[i].                                */

int ManytoMany_Con(
  void *,    /* address of the send buffer */
  int  *,   /* Number of elements send to each processor, This is an array of
               size PS */
  int  *,   /* the displacement of each send block from the beginning of 
               the sendbuf*/
  int   ,   /* the size , in bytes, of the elements */
  void *,   /* address of the receive buffer */
  int  *,   /* No. of elements receive from each processor. This is an array of
               size PS */
  int  *,   /* the displacement of each recv block from the beginning of the
               recvbuf */
  int   ,   /* the main algorithm number */
  int   ,   /* the first stage algorithm, in case of two stage algorithm*/
  int   ); /* the second stage algorithm, in case of two stage algorithm */ 
 

/*--  LP_Xor : Implementation of Many-to-Many with contigous buffer  ---*/
int LP_Many_Con(
  void *,    /* address of the send buffer */
  int  *,   /* Number of elements send to each processor, This is an array of
               size PS */
  int  *,   /* the displacement of each send block from the beginning of 
               the sendbuf*/
  int   ,   /* the size , in bytes, of the elements */
  void *,   /* address of the receive buffer */
  int  *,   /* No. of elements receive from each processor. This is an array of
               size PS */
  int  *);   /* the displacement of each recv block from the beginning of the
               recvbuf */

/*--  Async. Comm. : Implementation of Many-to-Many with contigous buffer ---*/
int AC_Many_Con(
 void *,    /* address of the send buffer */
  int  *,   /* Number of elements send to each processor, This is an array of
               size PS */
  int  *,   /* the displacement of each send block from the beginning of 
               the sendbuf*/
  int   ,   /* the size , in bytes, of the elements */
  void *,   /* address of the receive buffer */
  int  *,   /* No. of elements receive from each processor. This is an array of
               size PS */
  int  *);   /* the displacement of each recv block from the beginning of the
               recvbuf */


/*--- Distributed Scheduling : Implementation of Many-to-Many with contigous
                                buffer ----*/
int  DS_Many_Con(
 void *,    /* address of the send buffer */
  int  *,   /* Number of elements send to each processor, This is an array of
               size PS */
  int  *,   /* the displacement of each send block from the beginning of 
               the sendbuf*/
  int   ,   /* the size , in bytes, of the elements */
  void *,   /* address of the receive buffer */
  int  *,   /* No. of elements receive from each processor. This is an array of
               size PS */
  int  *);   /* the displacement of each recv block from the beginning of the
               recvbuf */

/*-- Two Stage Alg. : Implementation of Many-to-Many with contigous buffer -*/
int Two_Stage_Many_Con(
  void *,    /* address of the send buffer */
  int  *,   /* Number of elements send to each processor, This is an array of
               size PS */
  int  *,   /* the displacement of each send block from the beginning of
               the sendbuf*/
  int   ,   /* the size , in bytes, of the elements */
  void *,   /* address of the receive buffer */
  int  *,   /* No. of elements receive from each processor. This is an array of
               size PS */
  int  *,   /* the displacement of each recv block from the beginning of the
               recvbuf */
  int   ,
  int  ); 


/*******************  Global Operations routines prototypes ***************/
int  Intialization();
int GenBarrier();

void Set_Global_Or(int);
int  Get_Global_Or(void);

int Gen_reduce_max_int(int);
int Gen_prefix_sum_int(int);

void segement_scan_v(int *, int, int *, int);

void Gen_global_sum_v(int *, int *, int);

void Gen_global_prefix_sum_v(int *, int *,int );
/*******************  Point-to-Point rountines prototypes   ***************/
/*---Generic Asy. Send  : returns message id ---*/
long GenAsySend(
   void *,   /* address of the send buffer */
   int  ,    /* number of bytes */
   int  ,   /* destination address */
   int  );   /* message tag */

/*---Generic syn. Send ---*/
int GenSynSend(
   void *,   /* address of the send buffer */
   int  ,    /* number of bytes */
   int  ,   /* destination address */
   int  );   /* message tag */

/*---Generic Asy. receive : returns message id ---*/
long GenAsyReceive(
   void *,   /* address of the send buffer */
   int  ,    /* number of bytes */
   int  ,   /* destination address */
   int  );   /* message tag */

/*---Generic syn. Receive ---*/
int GenSynReceive(
   void *,   /* address of the receive buffer */
   int  ,    /* number of bytes */
   int  ,   /* source address */
   int  );   /* message tag */

/*---- Generic Handler for Asyn. send/receive */
void   GenAsyHanlde(long);

/*--------------------- Active Messages Routines Prototypes ------------*/
      /*---   RPorts  Routines   ------*/

int Allocate_rport(int);

int Free_rport(int);

int Get_rport_byte_counter(int);

void Set_rport_byte_counter(int,int);

void Set_rport_handler(int, void *);

void Set_rport_address(int, void *);

      /*----  Polling Routines  ------*/
void Gen_Poll(void);

void Gen_Poll_until(int *,int);

     /*----  Sending  Active  Message  -----*/

/*int Gen_request(int,void *,int,int,int,int);

int Gen_reply(int,void *,int,int,int,int);
*/

    /*----   Sending Block of Data   -----*/
int Gen_send_rport(
  int,          /* destination */
  int,          /* rport number */
  int,          /* offset */
  void *,       /* source address */
  void *,       /* dest buf. address */
  int,          /* Number of elements */
  int );        /* element size , in bytes */




/*========================================================================*/
/*#include "collecti.c"  /* The collective communications routines *
#include "global.c"      /* The global operations routines *
#include "native.c"      /* Point-to-Point routines *
#include "rarw_LDB.c"        /* Random Access Routines */
/*int Expert_System()
{
  return(1);
} */

